unit FSWdmset;
//
//	FSWSetDatamodule
//
//	This component is used to link datamodules to forms at run-time. This makes
//	using datamodules with MDI Child forms easy.
//
//	Usage:
//
//  	1) Drop an FSWSetDataModule component on your form.
//
//  	2) Create an instance of your datamodule at run-time-
//			add to private variables:
//       	fMyMDIDatamodule: TMyMDIDatamodule;
//       add to FormCreate:
//        	fMyMDIDatamodule := TMyMDIDatamodule.Create(Self);
//				FSWSetDataModule1.DataModule := TDataModule(fMyMDIDataModule);
//
// 	3) If you are using Infopower components, uncomment the next line.
//				This will let the component handle the LookupTable property.
//
//{$define UseInfopower}
//
//	That's it.
//
//Author:
//
//  Ken Bailey
//  Falmouth Software Works Ltd.
//  202 U.S. Route 1, Suite 215
//  Falmouth, ME
//
//  Web Site: 	www.falmouth-software.com
//  email:	kbailey@falmouth-software.com
//
//  *PLEASE NOTE*  Before making any bug reports please first verify you are
//                 using the latest version by checking my home page.
//
//  You are free to use, modify, and distribute this code subject to following
//  restrictions:
//  - If you copy portions of my code or get ideas from my code, please give
//    me credit.
//  - Do not release modified versions of this source code unless it has
//    significant differences and is also freeware. If you've made any changes
//    that you feel should have been there in the first place, feel free to
//    submit them to me so I can try to incorporate them.
//
//Credits:
//  This component is based on code from a European Delphi magazine. Unfortunatly,
//  due to a crash of my email program, I have lost the original reference and
//  the name of the person who sent it to me. Information leading to the
//  rectification of this situation would be greatly appreciated.
//

interface

uses 	DBTables, DB, Classes, Forms, Controls;

Type

	TFSWSetDataSource = class(TComponent)
	private
    { Private declarations }
     	fDataModule				:	TDataModule;
      procedure setDataModule(dm : TDataModule);
   public
    { Public declarations }
   published
    { Published declarations }
     property DataModule: TDataModule read fDataModule write setDataModule;
	end;

procedure Register;

implementation

uses SysUtils, Windows, Typinfo;

procedure TFSWSetDataSource.setDataModule(dm : TDataModule);

var
   c        : TComponent;
   PropInfo : PPropInfo;
   ds       : TDatasource;
   i 			: integer;
   tb			: TDataset;

begin
	fDataModule := dm;
//
//		Scan through the Form's (Owner's) components and look for
//			datasource properties. When a datasource property is found,
//			check for an instance of then component on the datamodule. If it
//			exists then change the datasource property to the new instance.
//
   with Owner do
      for i := 0 to ComponentCount - 1 do
      begin
//		Does this component have a dataset property?
         PropInfo := GetPropInfo( Components[ i ].ClassInfo, 'DataSource' );
         if Assigned( PropInfo ) then
            try
//		Is the datasource property assigned?
               c  := TComponent(GetOrdProp(Components[ i ], PropInfo));
					if assigned(c) then
					begin
//		Is there a component on the new datamodule with the assigned name?
						ds := TDataSource(dm.FindComponent( c.Name ));
						if Assigned( ds ) then
//		Set the datasource property to the new component
							SetOrdProp( Components[ i ], PropInfo, Longint(ds) );
					end;
				except
               on EAccessViolation do
                  //Nothing;
            end;	{Try...except}
{$ifdef UseInfopower}
//
//		Handle the Infopower LookupTable property.
//
//		Does this component have a lookuptable property?
         PropInfo := GetPropInfo( Components[ i ].ClassInfo, 'LookupTable' );
         if Assigned( PropInfo ) then
            try
//		Is the lookuptable property assigned?
               tb := TDataset(GetOrdProp( Components[ i ], PropInfo ));
               if assigned(tb) then
               begin
//		Is there a component on the new datamodule with the assigned name?
               	tb := TDataset(dm.FindComponent( tb.Name ));
               	if Assigned( tb ) then
//		Set the lookuptable property to the new component
                  	SetOrdProp( Components[ i ], PropInfo, Longint(tb) );
               end;
            except
               on EAccessViolation do
            end;	{Try...except}
{$endif}
      end;	{for i := 0 to ComponentCount - 1 do}
end;

procedure Register;
begin
  RegisterComponents('FSW', [TFSWSetDataSource]);
end;

end.
